"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createSelectMiddleware = exports.createIncludeMiddleware = exports.createWhereMiddleware = exports.createAggregateMiddleware = exports.createCountMiddleware = exports.createGroupByMiddleware = exports.createFindManyMiddleware = exports.createFindFirstOrThrowMiddleware = exports.createFindFirstMiddleware = exports.createFindUniqueOrThrowMiddleware = exports.createFindUniqueMiddleware = exports.createUpsertMiddleware = exports.createUpdateManyMiddleware = exports.createUpdateMiddleware = exports.createDeleteManyMiddleware = exports.createDeleteMiddleware = void 0;
const client_1 = require("@prisma/client");
const nestedReads_1 = require("./utils/nestedReads");
const resultFiltering_1 = require("./utils/resultFiltering");
const uniqueFieldsByModel = {};
const uniqueIndexFieldsByModel = {};
client_1.Prisma.dmmf.datamodel.models.forEach((model) => {
    // add unique fields derived from indexes
    const uniqueIndexFields = [];
    model.uniqueFields.forEach((field) => {
        uniqueIndexFields.push(field.join("_"));
    });
    uniqueIndexFieldsByModel[model.name] = uniqueIndexFields;
    // add id field and unique fields from @unique decorator
    const uniqueFields = [];
    model.fields.forEach((field) => {
        if (field.isId || field.isUnique) {
            uniqueFields.push(field.name);
        }
    });
    uniqueFieldsByModel[model.name] = uniqueFields;
});
/* Delete middleware */
function createDeleteParams(params, { field, createValue }) {
    var _a, _b;
    if (!params.model ||
        // do nothing for delete: false
        (typeof params.args === "boolean" && !params.args) ||
        // do nothing for root delete without where to allow Prisma to throw
        (!params.scope && !((_a = params.args) === null || _a === void 0 ? void 0 : _a.where))) {
        return params;
    }
    if (typeof params.args === "boolean") {
        return {
            ...params,
            action: "update",
            args: {
                __passUpdateThrough: true,
                [field]: createValue(true),
            },
        };
    }
    return {
        ...params,
        action: "update",
        args: {
            where: ((_b = params.args) === null || _b === void 0 ? void 0 : _b.where) || params.args,
            data: {
                [field]: createValue(true),
            },
        },
    };
}
function createDeleteMiddleware(config) {
    return function deleteMiddleware(params, next) {
        return next(createDeleteParams(params, config));
    };
}
exports.createDeleteMiddleware = createDeleteMiddleware;
/* DeleteMany middleware */
function createDeleteManyParams(params, config) {
    var _a;
    if (!params.model)
        return params;
    const where = ((_a = params.args) === null || _a === void 0 ? void 0 : _a.where) || params.args;
    return {
        ...params,
        action: "updateMany",
        args: {
            where: {
                ...where,
                [config.field]: config.createValue(false),
            },
            data: {
                [config.field]: config.createValue(true),
            },
        },
    };
}
function createDeleteManyMiddleware(config) {
    return function deleteManyMiddleware(params, next) {
        return next(createDeleteManyParams(params, config));
    };
}
exports.createDeleteManyMiddleware = createDeleteManyMiddleware;
/* Update middleware */
function createUpdateParams(params, config) {
    var _a, _b, _c, _d;
    if (((_a = params.scope) === null || _a === void 0 ? void 0 : _a.relations) &&
        !params.scope.relations.to.isList &&
        !config.allowToOneUpdates &&
        !((_b = params.args) === null || _b === void 0 ? void 0 : _b.__passUpdateThrough)) {
        throw new Error(`prisma-soft-delete-middleware: update of model "${params.model}" through "${(_c = params.scope) === null || _c === void 0 ? void 0 : _c.parentParams.model}.${params.scope.relations.to.name}" found. Updates of soft deleted models through a toOne relation is not supported as it is possible to update a soft deleted record.`);
    }
    // remove __passUpdateThrough from args
    if ((_d = params.args) === null || _d === void 0 ? void 0 : _d.__passUpdateThrough) {
        delete params.args.__passUpdateThrough;
    }
    return params;
}
function createUpdateMiddleware(config) {
    return function updateMiddleware(params, next) {
        return next(createUpdateParams(params, config));
    };
}
exports.createUpdateMiddleware = createUpdateMiddleware;
/* UpdateMany middleware */
function createUpdateManyParams(params, config) {
    var _a, _b, _c;
    // do nothing if args are not defined to allow Prisma to throw an error
    if (!params.args)
        return params;
    return {
        ...params,
        args: {
            ...params.args,
            where: {
                ...(_a = params.args) === null || _a === void 0 ? void 0 : _a.where,
                // allow overriding the deleted field in where
                [config.field]: ((_c = (_b = params.args) === null || _b === void 0 ? void 0 : _b.where) === null || _c === void 0 ? void 0 : _c[config.field]) || config.createValue(false),
            },
        },
    };
}
function createUpdateManyMiddleware(config) {
    return function updateManyMiddleware(params, next) {
        return next(createUpdateManyParams(params, config));
    };
}
exports.createUpdateManyMiddleware = createUpdateManyMiddleware;
/* Upsert middleware */
function createUpsertMiddleware(_) {
    return function upsertMiddleware(params, next) {
        var _a, _b;
        if (((_a = params.scope) === null || _a === void 0 ? void 0 : _a.relations) && !params.scope.relations.to.isList) {
            throw new Error(`prisma-soft-delete-middleware: upsert of model "${params.model}" through "${(_b = params.scope) === null || _b === void 0 ? void 0 : _b.parentParams.model}.${params.scope.relations.to.name}" found. Upserts of soft deleted models through a toOne relation is not supported as it is possible to update a soft deleted record.`);
        }
        return next(params);
    };
}
exports.createUpsertMiddleware = createUpsertMiddleware;
/* FindUnique middleware helpers */
function validateFindUniqueParams(params, config) {
    var _a;
    const uniqueIndexFields = uniqueIndexFieldsByModel[params.model || ""] || [];
    const uniqueIndexField = Object.keys(((_a = params.args) === null || _a === void 0 ? void 0 : _a.where) || {}).find((key) => uniqueIndexFields.includes(key));
    // when unique index field is found it is not possible to use findFirst.
    // Instead warn the user that soft-deleted models will not be excluded from
    // this query unless warnForUniqueIndexes is false.
    if (uniqueIndexField && !config.allowCompoundUniqueIndexWhere) {
        throw new Error(`prisma-soft-delete-middleware: query of model "${params.model}" through compound unique index field "${uniqueIndexField}" found. Queries of soft deleted models through a unique index are not supported. Set "allowCompoundUniqueIndexWhere" to true to override this behaviour.`);
    }
}
function shouldPassFindUniqueParamsThrough(params, config) {
    var _a, _b;
    const uniqueFields = uniqueFieldsByModel[params.model || ""] || [];
    const uniqueIndexFields = uniqueIndexFieldsByModel[params.model || ""] || [];
    const uniqueIndexField = Object.keys(((_a = params.args) === null || _a === void 0 ? void 0 : _a.where) || {}).find((key) => uniqueIndexFields.includes(key));
    // pass through invalid args so Prisma throws an error
    return (
    // findUnique must have a where object
    !((_b = params.args) === null || _b === void 0 ? void 0 : _b.where) ||
        typeof params.args.where !== "object" ||
        // where object must have at least one defined unique field
        !Object.entries(params.args.where).some(([key, val]) => (uniqueFields.includes(key) || uniqueIndexFields.includes(key)) &&
            typeof val !== "undefined") ||
        // pass through if where object has a unique index field and allowCompoundUniqueIndexWhere is true
        !!(uniqueIndexField && config.allowCompoundUniqueIndexWhere));
}
/* FindUnique middleware */
function createFindUniqueParams(params, config) {
    var _a;
    if (shouldPassFindUniqueParamsThrough(params, config)) {
        return params;
    }
    validateFindUniqueParams(params, config);
    return {
        ...params,
        action: "findFirst",
        args: {
            ...params.args,
            where: {
                ...(_a = params.args) === null || _a === void 0 ? void 0 : _a.where,
                [config.field]: config.createValue(false),
            },
        },
    };
}
function createFindUniqueMiddleware(config) {
    return function findUniqueMiddleware(params, next) {
        return next(createFindUniqueParams(params, config));
    };
}
exports.createFindUniqueMiddleware = createFindUniqueMiddleware;
/* FindUniqueOrThrow middleware */
function createFindUniqueOrThrowParams(params, config) {
    var _a;
    if (shouldPassFindUniqueParamsThrough(params, config)) {
        return params;
    }
    validateFindUniqueParams(params, config);
    return {
        ...params,
        action: "findFirstOrThrow",
        args: {
            ...params.args,
            where: {
                ...(_a = params.args) === null || _a === void 0 ? void 0 : _a.where,
                [config.field]: config.createValue(false),
            },
        },
    };
}
function createFindUniqueOrThrowMiddleware(config) {
    return function findUniqueMiddleware(params, next) {
        return next(createFindUniqueOrThrowParams(params, config));
    };
}
exports.createFindUniqueOrThrowMiddleware = createFindUniqueOrThrowMiddleware;
/* FindFirst middleware */
function createFindFirstParams(params, config) {
    var _a, _b, _c;
    return {
        ...params,
        action: "findFirst",
        args: {
            ...params.args,
            where: {
                ...(_a = params.args) === null || _a === void 0 ? void 0 : _a.where,
                // allow overriding the deleted field in where
                [config.field]: ((_c = (_b = params.args) === null || _b === void 0 ? void 0 : _b.where) === null || _c === void 0 ? void 0 : _c[config.field]) || config.createValue(false),
            },
        },
    };
}
function createFindFirstMiddleware(config) {
    return function findFirst(params, next) {
        return next(createFindFirstParams(params, config));
    };
}
exports.createFindFirstMiddleware = createFindFirstMiddleware;
/* FindFirst middleware */
function createFindFirstOrThrowParams(params, config) {
    var _a, _b, _c;
    return {
        ...params,
        action: "findFirstOrThrow",
        args: {
            ...params.args,
            where: {
                ...(_a = params.args) === null || _a === void 0 ? void 0 : _a.where,
                // allow overriding the deleted field in where
                [config.field]: ((_c = (_b = params.args) === null || _b === void 0 ? void 0 : _b.where) === null || _c === void 0 ? void 0 : _c[config.field]) || config.createValue(false),
            },
        },
    };
}
function createFindFirstOrThrowMiddleware(config) {
    return function findFirstOrThrow(params, next) {
        return next(createFindFirstOrThrowParams(params, config));
    };
}
exports.createFindFirstOrThrowMiddleware = createFindFirstOrThrowMiddleware;
/* FindMany middleware */
function createFindManyParams(params, config) {
    var _a, _b, _c;
    return {
        ...params,
        action: "findMany",
        args: {
            ...params.args,
            where: {
                ...(_a = params.args) === null || _a === void 0 ? void 0 : _a.where,
                // allow overriding the deleted field in where
                [config.field]: ((_c = (_b = params.args) === null || _b === void 0 ? void 0 : _b.where) === null || _c === void 0 ? void 0 : _c[config.field]) || config.createValue(false),
            },
        },
    };
}
function createFindManyMiddleware(config) {
    return function findManyMiddleware(params, next) {
        return next(createFindManyParams(params, config));
    };
}
exports.createFindManyMiddleware = createFindManyMiddleware;
/*GroupBy middleware */
function createGroupByParams(params, config) {
    var _a, _b, _c;
    return {
        ...params,
        action: "groupBy",
        args: {
            ...params.args,
            where: {
                ...(_a = params.args) === null || _a === void 0 ? void 0 : _a.where,
                // allow overriding the deleted field in where
                [config.field]: ((_c = (_b = params.args) === null || _b === void 0 ? void 0 : _b.where) === null || _c === void 0 ? void 0 : _c[config.field]) || config.createValue(false),
            },
        },
    };
}
function createGroupByMiddleware(config) {
    return function groupByMiddleware(params, next) {
        return next(createGroupByParams(params, config));
    };
}
exports.createGroupByMiddleware = createGroupByMiddleware;
/* Count middleware */
function createCountParams(params, config) {
    const args = params.args || {};
    const where = args.where || {};
    return {
        ...params,
        args: {
            ...args,
            where: {
                ...where,
                // allow overriding the deleted field in where
                [config.field]: where[config.field] || config.createValue(false),
            },
        },
    };
}
function createCountMiddleware(config) {
    return function countMiddleware(params, next) {
        return next(createCountParams(params, config));
    };
}
exports.createCountMiddleware = createCountMiddleware;
/* Aggregate middleware */
function createAggregateParams(params, config) {
    const args = params.args || {};
    const where = args.where || {};
    return {
        ...params,
        args: {
            ...args,
            where: {
                ...where,
                // allow overriding the deleted field in where
                [config.field]: where[config.field] || config.createValue(false),
            },
        },
    };
}
function createAggregateMiddleware(config) {
    return function aggregateMiddleware(params, next) {
        return next(createAggregateParams(params, config));
    };
}
exports.createAggregateMiddleware = createAggregateMiddleware;
/* Where middleware */
function createWhereParams(params, config) {
    var _a;
    // customise list queries with every modifier unless the deleted field is set
    if (((_a = params.scope) === null || _a === void 0 ? void 0 : _a.modifier) === "every" && !params.args[config.field]) {
        return {
            ...params,
            args: {
                OR: [
                    { [config.field]: { not: config.createValue(false) } },
                    params.args,
                ],
            },
        };
    }
    return {
        ...params,
        args: {
            ...params.args,
            [config.field]: params.args[config.field] || config.createValue(false),
        },
    };
}
function createWhereMiddleware(config) {
    return function whereMiddleware(params, next) {
        if (!params.scope)
            return next(params);
        return next(createWhereParams(params, config));
    };
}
exports.createWhereMiddleware = createWhereMiddleware;
/* Include middleware */
function createIncludeParams(params, config) {
    var _a, _b, _c, _d, _e, _f, _g;
    // includes of toOne relation cannot filter deleted records using params
    // instead ensure that the deleted field is selected and filter the results
    if (((_b = (_a = params.scope) === null || _a === void 0 ? void 0 : _a.relations) === null || _b === void 0 ? void 0 : _b.to.isList) === false) {
        if (((_c = params.args) === null || _c === void 0 ? void 0 : _c.select) && !((_d = params.args) === null || _d === void 0 ? void 0 : _d.select[config.field])) {
            return (0, nestedReads_1.addDeletedToSelect)(params, config);
        }
        return params;
    }
    return {
        ...params,
        args: {
            ...params.args,
            where: {
                ...(_e = params.args) === null || _e === void 0 ? void 0 : _e.where,
                // allow overriding the deleted field in where
                [config.field]: ((_g = (_f = params.args) === null || _f === void 0 ? void 0 : _f.where) === null || _g === void 0 ? void 0 : _g[config.field]) || config.createValue(false),
            },
        },
    };
}
function createIncludeMiddleware(config) {
    return async function includeMiddleware(params, next) {
        var _a;
        const updatedParams = createIncludeParams(params, config);
        const deletedFieldAdded = typeof updatedParams.args === "object" &&
            ((_a = updatedParams.args) === null || _a === void 0 ? void 0 : _a.__deletedFieldAdded);
        if (deletedFieldAdded) {
            delete updatedParams.args.__deletedFieldAdded;
        }
        const result = await next(updatedParams);
        if ((0, resultFiltering_1.shouldFilterDeletedFromReadResult)(params, config)) {
            const filteredResults = (0, resultFiltering_1.filterSoftDeletedResults)(result, config);
            if (deletedFieldAdded) {
                (0, nestedReads_1.stripDeletedFieldFromResults)(filteredResults, config);
            }
            return filteredResults;
        }
        return result;
    };
}
exports.createIncludeMiddleware = createIncludeMiddleware;
/* Select middleware */
function createSelectParams(params, config) {
    var _a, _b, _c, _d, _e, _f, _g;
    // selects in includes are handled by createIncludeParams
    if (((_a = params.scope) === null || _a === void 0 ? void 0 : _a.parentParams.action) === "include") {
        return params;
    }
    // selects of toOne relation cannot filter deleted records using params
    if (((_c = (_b = params.scope) === null || _b === void 0 ? void 0 : _b.relations) === null || _c === void 0 ? void 0 : _c.to.isList) === false) {
        if (((_d = params.args) === null || _d === void 0 ? void 0 : _d.select) && !params.args.select[config.field]) {
            return (0, nestedReads_1.addDeletedToSelect)(params, config);
        }
        return params;
    }
    return {
        ...params,
        args: {
            ...params.args,
            where: {
                ...(_e = params.args) === null || _e === void 0 ? void 0 : _e.where,
                // allow overriding the deleted field in where
                [config.field]: ((_g = (_f = params.args) === null || _f === void 0 ? void 0 : _f.where) === null || _g === void 0 ? void 0 : _g[config.field]) || config.createValue(false),
            },
        },
    };
}
function createSelectMiddleware(config) {
    return async function selectMiddleware(params, next) {
        var _a;
        const updatedParams = createSelectParams(params, config);
        const deletedFieldAdded = typeof updatedParams.args === "object" &&
            ((_a = updatedParams.args) === null || _a === void 0 ? void 0 : _a.__deletedFieldAdded);
        if (deletedFieldAdded) {
            delete updatedParams.args.__deletedFieldAdded;
        }
        const result = await next(updatedParams);
        if ((0, resultFiltering_1.shouldFilterDeletedFromReadResult)(params, config)) {
            const filteredResults = (0, resultFiltering_1.filterSoftDeletedResults)(result, config);
            if (deletedFieldAdded) {
                (0, nestedReads_1.stripDeletedFieldFromResults)(filteredResults, config);
            }
            return filteredResults;
        }
        return result;
    };
}
exports.createSelectMiddleware = createSelectMiddleware;
